module ts4600_top(
inout [7:0] nand_dat_pad,
input nand_rdn_pad,
input nand_wrn_pad,
input nand_ale_pad,
output nand_rdy_pad,
input nand_cs0n_pad, 

output [1:0] sd_fault_led,

output led_grnn_pad,
output led_redn_pad,

inout [69:0] dio_pad,

input fpga_resetn_pad,
input fpga_12mhz_pad,

output cpu_reset_pad,
output hard_rebootn_pad,
output offbd_resetn_pad,

output batt_loadn_pad,
output [2:0] fpga_irq_pad,

output [1:0] boot_strap_pad
);

parameter spi_opt = 1'b1;
parameter mwin_opt = 1'b1;
parameter xbus_opt = 1'b1;
parameter touchscreen_opt = 1'b1;
/* software currently requires these to be enabled/disabled contiguously. */
parameter xuart0_opt = 1'b1;
parameter xuart1_opt = 1'b1;
parameter xuart2_opt = 1'b1;
parameter xuart3_opt = 1'b1;
parameter xuart4_opt = 1'b1;
parameter xuart5_opt = 1'b0;
parameter xuart6_opt = 1'b0;
parameter xuart7_opt = 1'b0;

/****************************************************************************
* Boilerplate FPGA configuration (clocks, pll, reset, etc.)
*****************************************************************************/
wire wb_clk;
assign wb_clk = pll_75mhz;
reg rst/* synthesis syn_preserve=1 */;
reg rstn/* synthesis syn_preserve=1 */; 
wire wb_rst;
assign wb_rst = rst;
reg [31:0] timer;
reg cpu_reset;
reg [1:0] mode/* synthesis syn_preserve=1 */;
reg timeout;
wire [1:0] sd_led;
reg [1:0] mode_q;
wire [69:0] dio_i;

assign cpu_reset_pad = cpu_reset;
assign hard_rebootn_pad = sc_reboot ? 1'b0 : 1'bz;
assign boot_strap_pad = timeout ? 2'bzz : mode;
assign offbd_resetn_pad = !cpu_reset;
assign sd_fault_led = ~(sd_led);

wire lock, pll_75mhz, pll_100mhz;
pll0 clkgen (
  .CLK(fpga_12mhz_pad),
  //.CLKOP( ),
  .CLKOK(pll_75mhz),
  .CLKOK2(pll_100mhz),
  .LOCK(lock)
);

always @(posedge internal_osc_o or negedge fpga_resetn_pad) begin
  if(!fpga_resetn_pad) begin
    cpu_reset <= 1'b1;
    mode <= 2'b0;
    timer <= 32'd0;
    rstn <= 1'b0;
    timeout <= 1'b0;
  end else begin
    rstn <= 1'b1;
    timer <= timer + 1'b1;
    if(timer > 32'd64) cpu_reset <= 1'b0;
    if(timer > 32'd543000) timeout <= 1'b1;
    if(cpu_reset) begin
      mode <= {2{dio_i[20]}};
    end
  end
end

always @(posedge pll_75mhz or negedge rstn) begin
  if(!rstn) begin
    rst <= 1'b1;
  end else if(lock) begin
    rst <= 1'b0;
    mode_q <= mode;
  end
end

wire clk_25mhz, clk_12p5mhz;
reg clk_14p28mhz;
reg [3:0] evencnt;
reg [2:0] state_14mhz;
assign clk_25mhz = evencnt[1];
assign clk_12p5mhz = evencnt[2];
always @(posedge pll_100mhz or posedge rst) begin
  if(rst) begin
    evencnt <= 4'b0;
    clk_14p28mhz <= 1'b0;
    state_14mhz <= 2'b0;
  end else begin
    evencnt <= evencnt + 1'b1;
    if(state_14mhz == 3'd6) begin
      state_14mhz <= 3'd0;
      clk_14p28mhz <= 1'b0;
    end else state_14mhz <= state_14mhz + 1'b1;
    if(state_14mhz == 3'd3) clk_14p28mhz <= 1'b1;
  end
end

//assign eth_right_led_pad = dio_oe[0] ? dio_o[0] : !eth_100mbit_led_pad;
assign led_grnn_pad = !led_grn;
assign led_redn_pad = !led_red;
assign fpga_irq_pad[2] = 1'b0;

/****************************************************************************
* NBUS interface, NAND bus like interface
*****************************************************************************/
wire cpuwbm_cyc_o, cpuwbm_stb_o, cpuwbm_we_o;
reg [15:0] cpuwbm_dat_i;
wire [15:0] cpuwbm_dat_o;
wire [7:0] cpuwbm_adr_o;
reg cpuwbm_ack_i;
wire [1:0] cpuwbm_sel_o;

wire nand_dat_oe;
wire [7:0] nand_dat_o;
wire [7:0] nand_dat_i;

genvar i;
generate
  for(i = 0; i < 8; i = i+1) begin: nandbus
    BB nand_dat_bb(.I(nand_dat_o[i]), .T(!nand_dat_oe), .B(nand_dat_pad[i]), .O(nand_dat_i[i]));
  end
endgenerate
mxsnandbridge nand_bus (
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),
  .wbm_cyc_o(cpuwbm_cyc_o),
  .wbm_stb_o(cpuwbm_stb_o),
  .wbm_dat_o(cpuwbm_dat_o),
  .wbm_adr_o(cpuwbm_adr_o),
  .wbm_we_o(cpuwbm_we_o),
  .wbm_dat_i(cpuwbm_dat_i),
  .wbm_ack_i(cpuwbm_ack_i),
  .wbm_sel_o(cpuwbm_sel_o),
  
  .nand_dat_i(nand_dat_i),
  .nand_dat_o(nand_dat_o),
  .nand_dat_oe_o(nand_dat_oe),
  .nand_stbn_i(nand_wrn_pad),
  .nand_rdnwr_i(nand_rdn_pad),
  .nand_csn_i(nand_cs0n_pad),
  .nand_ale_i(nand_ale_pad),
  
	.nand_rdyn_o(nand_rdy_pad)
);

/****************************************************************************
 * TS-BLOCKRAM interface, port to wishbone and NBUS
 ****************************************************************************/
wire ramwbs1_cyc_i, ramwbs1_stb_i, ramwbs1_we_i, ramwbs1_ack_o;
wire [31:0] ramwbs1_dat_i, ramwbs1_dat_o;
wire [31:0] ramwbs1_adr_i;
wire ramwbs2_cyc_i, ramwbs2_stb_i, ramwbs2_we_i, ramwbs2_ack_o;
wire [31:0] ramwbs2_dat_i, ramwbs2_dat_o;
wire [31:0] ramwbs2_adr_i;
wire [3:0] ramwbs2_sel_i, ramwbs1_sel_i;

blockram_8kbyte sramcore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  /* To XUART */
  .wb1_cyc_i(ramwbs1_cyc_i),
  .wb1_stb_i(ramwbs1_stb_i),
  .wb1_we_i(ramwbs1_we_i),
  .wb1_dat_i(ramwbs1_dat_i),
  .wb1_dat_o(ramwbs1_dat_o),
  .wb1_adr_i(ramwbs1_adr_i),
  .wb1_ack_o(ramwbs1_ack_o),
  .wb1_sel_i(ramwbs1_sel_i),

  /* To wb_memwindow */
  .wb2_cyc_i(ramwbs2_cyc_i),
  .wb2_stb_i(ramwbs2_stb_i),
  .wb2_we_i(ramwbs2_we_i),
  .wb2_dat_i(ramwbs2_dat_i),
  .wb2_dat_o(ramwbs2_dat_o),
  .wb2_adr_i(ramwbs2_adr_i),
  .wb2_ack_o(ramwbs2_ack_o),
  .wb2_sel_i(ramwbs2_sel_i)
);

/****************************************************************************
* XUART Super UARTS
*****************************************************************************/
wire xuwbs_ack_o;
reg xuwbs_en;
wire [31:0] uartwbm_adr_o;
wire [15:0] uartwbm_dat_o, xuwbs_dat_o;
wire [15:0] uartwbm_dat_i;
reg [7:0] xuart_rx;
wire [7:0] xuart_active;
wire [7:0] xuart_cts = dio_i[48:41];
wire [7:0] xuart_txen, xuart_tx;
reg [7:0] xuart_rx_q, xuart_cts_q;
wire xuart_irq;
assign fpga_irq_pad[0] = xuart_irq;
wire [7:0] xuart_disable = {!xuart7_opt[0], !xuart6_opt[0], !xuart5_opt[0],
  !xuart4_opt[0], !xuart3_opt[0], !xuart2_opt[0], !xuart1_opt[0],
  !xuart0_opt[0]};
wire xuart_opt = (xuart_disable[7:0] != 8'hff);
always @(posedge wb_clk) begin
  xuart_rx_q <= xuart_rx;
  xuart_cts_q <= xuart_cts;
end
assign ramwbs1_dat_i[31:0] = {uartwbm_dat_o[15:0], uartwbm_dat_o[15:0]};
assign ramwbs1_adr_i = {uartwbm_adr_o[15:2], 2'b00};
assign uartwbm_dat_i[15:0] =
  uartwbm_adr_o[1] ? ramwbs1_dat_o[31:16] : ramwbs1_dat_o[15:0];
assign ramwbs1_sel_i = uartwbm_adr_o[1] ? 4'b1100 : 4'b0011;
ts_xuart tsxuartcore(
  .wb_clk_i(wb_clk & xuart_opt),
  .wb_rst_i(wb_rst),

  .wbm_cyc_o(ramwbs1_cyc_i),
  .wbm_stb_o(ramwbs1_stb_i),
  .wbm_we_o(ramwbs1_we_i),
  .wbm_ack_i(ramwbs1_ack_o),
  .wbm_adr_o(uartwbm_adr_o),
  .wbm_dat_i(uartwbm_dat_i),
  .wbm_dat_o(uartwbm_dat_o),


  .wbs_cyc_i(cpuwbm_cyc_o && xuwbs_en),
  .wbs_stb_i(cpuwbm_stb_o && xuwbs_en),
  .wbs_we_i(cpuwbm_we_o),
  .wbs_adr_i(cpuwbm_adr_o),
  .wbs_dat_i(cpuwbm_dat_o),
  .wbs_dat_o(xuwbs_dat_o),
  .wbs_ack_o(xuwbs_ack_o),

  .baudclk_i(pll_100mhz),
  .cts_i(xuart_cts_q),
  .rx_i(xuart_rx_q),
  .tx_o(xuart_tx),
  .txen_o(xuart_txen),
  .active_o(xuart_active),
  .irq_o(xuart_irq),
  .uart_disable_i(xuart_disable)
);

/****************************************************************************
 * NBUS blockram window (for XUART 8kbyte memory access)
 ****************************************************************************/
reg ramwinwbs_en;
wire [15:0] ramwinwbs_dat_o;
wire ramwinwbs_ack_o;
wb_memwindow16to32 ramwincore(
  .wb_clk_i(wb_clk && xuart_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(cpuwbm_cyc_o && ramwinwbs_en),
  .wb_stb_i(cpuwbm_stb_o && ramwinwbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_dat_o(ramwinwbs_dat_o),
  .wb_ack_o(ramwinwbs_ack_o),

  .wbm_cyc_o(ramwbs2_cyc_i),
  .wbm_stb_o(ramwbs2_stb_i),
  .wbm_we_o(ramwbs2_we_i),
  .wbm_adr_o(ramwbs2_adr_i),
  .wbm_dat_o(ramwbs2_dat_i),
  .wbm_dat_i(ramwbs2_dat_o),
  .wbm_ack_i(ramwbs2_ack_o),
  .wbm_sel_o(ramwbs2_sel_i)
);

/****************************************************************************
 * General purpose memory window for external bus and any custom registers.
 ****************************************************************************/
wire mwinwbm_cyc_o, mwinwbm_stb_o, mwinwbm_we_o;
reg mwinwbm_ack_i;
wire [31:0] mwinwbm_adr_o;
wire [31:0] mwinwbm_dat_o;
reg [31:0] mwinwbm_dat_i;
wire [3:0] mwinwbm_sel_o;
reg mwinwbs_en;
wire [15:0] mwinwbs_dat_o;
wire mwinwbs_ack_o;
reg [1:0] mwinbuswidth;
wb_memwindow mwincore(
  .wb_clk_i(wb_clk && mwin_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(cpuwbm_cyc_o && mwinwbs_en),
  .wb_stb_i(cpuwbm_stb_o && mwinwbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_sel_i(cpuwbm_sel_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_dat_o(mwinwbs_dat_o),
  .wb_ack_o(mwinwbs_ack_o),

  .wbm_cyc_o(mwinwbm_cyc_o),
  .wbm_stb_o(mwinwbm_stb_o),
  .wbm_we_o(mwinwbm_we_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o),
  .wbm_dat_o(mwinwbm_dat_o),
  .wbm_dat_i(mwinwbm_dat_i),
  .wbm_ack_i(mwinwbm_ack_i),
  .wbm_buswidth_i(mwinbuswidth)
);


/****************************************************************************
 * External 16 bit muxed address/data bus
 ****************************************************************************/
reg xbuswbs_en;
wire [15:0] xbuswbs_dat_o;
wire xbuswbs_ack_o;

/* The first 64KB of memwindow space are for using the external bus in
 * 16 bit mode.  The next 64KB (adr_o[16] == 1) are for the external bus in
 * 8 bit mode.  For 8 bit mode, the memwindow core will emit 8 bit wishbone
 * cycles, but the external bus core expects 16 bit cycles with a wb_sel of
 * 2'b01 or 2'b10.  This is a sensible way of implementing the external bus
 * on non-sbus designs, and rather than modify it for this situation, I am
 * going to hook up an 8 to 16 bit adapter for the 8 bit space.  This will
 * allow maximum bandwidth for PC/104 due to memwindow optimization features.
 * Actually since the TS-81xx only supports 8 bit PC/104, maybe we should put
 * an 8 bit memwindow on all macrocontrollers for this purpose.
 * -- GP
 */

wire [15:0] xbuswbs_dat16_o;
wire [31:0] xbuswbs_adr_i;
wire [15:0] xbuswbs_dat_i;
wire [1:0]  xbuswbs_sel_i;
wb_8to16adapter adapter(
  .wbm_dat_o(mwinwbm_dat_o[15:0]),
  .wbm_dat_i(xbuswbs_dat_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o[1:0]),

  .wbs_dat_i(xbuswbs_dat_i),
  .wbs_dat_o(xbuswbs_dat16_o),
  .wbs_adr_i(xbuswbs_adr_i),
  .wbs_sel_i(xbuswbs_sel_i),

  .toggle8bit_i(mwinwbm_adr_o[16])
);
wire xbus_csn, xbus_dir, xbus_ale, xbus_bhen, xbus_mux_oe, xbus_cycle;
wire [15:0] xbus_dat;
wire [15:0] bus_config;
isamuxbus_bridge externalbus(
  .wb_clk_i(wb_clk & xbus_opt),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(mwinwbm_cyc_o && xbuswbs_en),
  .wb_stb_i(mwinwbm_stb_o && xbuswbs_en),
  .wb_adr_i(xbuswbs_adr_i),
  .wb_dat_i(xbuswbs_dat_i),
  .wb_sel_i(xbuswbs_sel_i),
  .wb_dat_o(xbuswbs_dat16_o),
  .wb_ack_o(xbuswbs_ack_o),
  .wb_we_i(mwinwbm_we_o),

  .bus_config_i(bus_config),
  .cs_8bit_i(mwinwbm_adr_o[16]),

  .csn_o(xbus_csn),
  .dir_o(xbus_dir),
  .ale_o(xbus_ale),
  .bhen_o(xbus_bhen),
  .waitn_i(dio_i[17]),
  .mux_i(dio_i[37:22]),
  .mux_o(xbus_dat),
  .mux_oe_o(xbus_mux_oe),
  .cycle_o(xbus_cycle)
);


/****************************************************************************
 * SPI controller logic
 ****************************************************************************/
reg spiwbs_en;
wire [15:0] spiwbs_dat_o;
wire spiwbs_ack_o;
wire scken;
wire [3:0] csn;
wire [3:0] si = {4{dio_i[67]}};
wire sck, hispeed;
wire so;
wb_spi #(
  .only8bit_opt(1'b1)
) spicore(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk & spi_opt),
  .wb_cyc_i(cpuwbm_cyc_o && spiwbs_en),
  .wb_stb_i(cpuwbm_stb_o && spiwbs_en),
  .wb_dat_o(spiwbs_dat_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_we_i(cpuwbm_we_o),
  .wb_ack_o(spiwbs_ack_o),
  .wb_sel_i(cpuwbm_sel_o),

  .scken_o(scken),
  .so_o(so),
  .si_i(si),
  .csn_o(csn),
  .sck_o(sck),
  .hispeed_o(hispeed)
);

/****************************************************************************
 * ADC core
 ****************************************************************************/

reg adcwbs_en;
wire [15:0] adcwbs_dat_o;
wire adcwbs_ack_o;
wire adc_scl, adc_sda, an_sel;
wire [1:0] an_sel_sel;
wire en_adc;

wb_mcp3428 adc(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk),
  .wb_cyc_i(cpuwbm_cyc_o && adcwbs_en),
  .wb_stb_i(cpuwbm_stb_o && adcwbs_en),
  .wb_dat_o(adcwbs_dat_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_we_i(cpuwbm_we_o),
  .wb_ack_o(adcwbs_ack_o),
  .wb_sel_i(cpuwbm_sel_o),

  .scl_o(adc_scl),
  .sda_o(adc_sda),
  .sda_i(dio_i[6]),
  .an_sel_o(an_sel),
  .an_sel_sel_o(an_sel_sel),
  .active_o(en_adc)
);

/****************************************************************************
 * Touchscreen core
 ****************************************************************************/

reg tswbs_en;
wire [15:0] tswbs_dat_o;
wire tswbs_ack;
wire tswbs_ack_o = tswbs_ack | !touchscreen_opt;
wire touch_mosi, touch_clk, touch_csn; // CS# is open drain for some reason
/* TS-8280 uses the pins that would normally be claimed by the wb_spi */
wire miso = (ts_opt == 2'b01) ? dio_i[35] : ((ts_opt == 2'b10) ? dio_i[67] : 1'bz);
wire penirq = (ts_opt == 2'b01) ? dio_i[33] : ((ts_opt == 2'b10) ? dio_i[36] : 1'bz);
touchscreen ts(
  .wb_clk_i(wb_clk & touchscreen_opt),
  .wb_rst_i(wb_rst | !touchscreen_opt),
  .wb_cyc_i(cpuwbm_cyc_o && tswbs_en),
  .wb_stb_i(cpuwbm_stb_o && tswbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_dat_o(tswbs_dat_o),
  .wb_ack_o(tswbs_ack),

  .spi_miso_i(miso),
  .spi_mosi_o(touch_mosi),
  .spi_clk_o(touch_clk),
  .ad7843_csn_o(touch_csn),
  .ad7843_penirqn_i(penirq)
);

/****************************************************************************
* DIO assignments and Syscon
*****************************************************************************/
reg scwbs_en;
wire led_grn;
wire led_red;
wire evgpio_irq;
assign fpga_irq_pad[1] = evgpio_irq;
wire [15:0] scwbs_dat_o;
wire scwbs_ack_o;

reg [69:0] dio_o, dio_oe;
wire [69:0] dio_o_sc, dio_oe_sc;
wire [7:0] tx_en;
wire [1:0] bbclk;
wire [1:0] ts_opt;
wire spi_lcd;

genvar y;
generate
  for(y = 0; y < 70; y = y+1) begin: dio
    BB dio_bb(.I(dio_o[y]), .T(!dio_oe[y]), .B(dio_pad[y]), .O(dio_i[y]));
  end
endgenerate
wire sc_reboot;
wire internal_osc_o;
integer x;
always @(*) begin
  for(x = 0; x < 70; x = x+1) begin
    dio_o[x] = dio_o_sc[x];
    dio_oe[x] = dio_oe_sc[x];
  end

  if(spi_opt && (csn != 4'hf)) begin
    if(spi_lcd) begin
      dio_oe[64:61] = 4'b1111;
      dio_o[64] = sck; //hispeed ? (pll_75mhz_shifted | !scken) : sck;
      dio_o[63] = so;
      dio_o[62] = csn[0];
      dio_o[61] = (csn[0] && csn[1]);
    end else begin
      dio_oe[68:65] = 4'b1011;
      dio_o[66] = sck; //hispeed ? (pll_75mhz_shifted | !scken) : sck;
      dio_o[65] = so;
      dio_o[68] = csn[0];
    end
  end

  if(en_adc) begin
    dio_o[7:6] = 2'b00;
    dio_oe[7] = !adc_scl;
    dio_oe[6] = !adc_sda;
    if(an_sel_sel == 2'b01) begin
      dio_o[8] = an_sel;
      dio_oe[8] = 1'b1;
    end else if (an_sel_sel == 2'b10) begin
       dio_o[32] = an_sel;
       dio_oe[32] = 1'b1;
    end
  end

  xuart_rx[0] = dio_i[54] | (tx_en[0] && xuart_txen[0]);
  if (xuart_active[0]) begin
    dio_oe[53] = 1'b1;
    dio_o[53] = xuart_tx[0];
    dio_oe[54] = 1'b0;
    if(tx_en[0]) begin
      dio_o[12] = xuart_txen[0];
      dio_oe[12] = 1'b1;
    end
  end
  xuart_rx[1] = dio_i[56] | (tx_en[1] && xuart_txen[1]);
  if (xuart_active[1]) begin
    dio_oe[55] = 1'b1;
    dio_o[55] = xuart_tx[1];
    dio_oe[56] = 1'b0;
    if(tx_en[1]) begin
      dio_o[8] = xuart_txen[1];
      dio_oe[8] = 1'b1;
    end
  end
  xuart_rx[2] = dio_i[58] | (tx_en[2] && xuart_txen[2]);
  if (xuart_active[2]) begin
    dio_oe[57] = 1'b1;
    dio_o[57] = xuart_tx[2];
    dio_oe[58] = 1'b0;
    if(tx_en[2]) begin
      dio_o[10] = xuart_txen[2];
      dio_oe[10] = 1'b1;
    end
  end
  xuart_rx[3] = dio_i[60] | (tx_en[3] && xuart_txen[3]);
  if (xuart_active[3]) begin
    dio_oe[59] = 1'b1;
    dio_o[59] = xuart_tx[3];
    dio_oe[60] = 1'b0;
    if(tx_en[3]) begin
      dio_o[13] = xuart_txen[3];
      dio_oe[13] = 1'b1;
    end
  end
  xuart_rx[4] = dio_i[62] | (tx_en[4] && xuart_txen[4]);
  if (xuart_active[4]) begin
    dio_o[61] = xuart_tx[4];
    dio_oe[61] = 1'b1;
    dio_oe[62] = 1'b0;
    if(tx_en[4]) begin
      dio_o[14] = xuart_txen[4];
      dio_oe[14] = 1'b1;
    end
  end
  xuart_rx[5] = dio_i[64] | (tx_en[5] && xuart_txen[5]);
  if (xuart_active[5]) begin
    dio_oe[63] = 1'b1;
    dio_o[63] = xuart_tx[5];
    dio_oe[64] = 1'b0;
    if(tx_en[5]) begin
      dio_o[7] = xuart_txen[5];
      dio_oe[7] = 1'b1;
    end
  end

  if(xbus_opt & bus_config[0]) begin
    dio_o[18] = xbus_bhen;
    dio_o[19] = xbus_csn;
    dio_o[20] = xbus_dir;
    dio_o[21] = !xbus_ale; // Pin is ALE#, but ale is active high internally
    dio_oe[21:17] = 5'b11110;
		dio_o[37:22] = xbus_dat[15:0];
    dio_oe[37:22] = xbus_mux_oe ? 16'hffff : 16'h0000;
  end

  /*The TS-8280 TS uses wb_spi pins, TS needs priority */
  if(ts_opt > 2'b0) begin
    if(ts_opt == 2'b01) begin
      dio_o[36] = touch_mosi;
      dio_o[34] = touch_csn;
      dio_o[37] = touch_clk;
      dio_oe[37:34] = 4'b1101;
    end else if(ts_opt == 2'b10) begin
      dio_o[65] = touch_mosi;
      dio_o[68] = touch_csn;
      dio_o[66] = touch_clk;
      dio_oe[68:65] = 4'b1011;
    end
  end

  case(bbclk)
    2'b01: begin
      dio_oe[3] = 1'b1;
      dio_o[3] = clk_12p5mhz;
    end
    2'b10: begin
      dio_oe[3] = 1'b1;
      dio_o[3] = clk_14p28mhz;
    end
    2'b11: begin
      dio_oe[37] = 1'b1;
      dio_o[37] = clk_25mhz;
    end
  endcase
end


syscon mxs_syscon(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(cpuwbm_cyc_o && scwbs_en),
  .wb_stb_i(cpuwbm_stb_o && scwbs_en),
  .wb_we_i(cpuwbm_we_o),
  .wb_adr_i(cpuwbm_adr_o),
  .wb_dat_i(cpuwbm_dat_o),
  .wb_sel_i(cpuwbm_sel_o),
  .wb_dat_o(scwbs_dat_o),
  .wb_ack_o(scwbs_ack_o),

  .dio_i(dio_i),
  .dio_oe_o(dio_oe_sc),
  .dio_o(dio_o_sc),
  .evgpio_irq_o(evgpio_irq),

  .cpu_uart_rxd_o(debug_rxd_pad),
  .cpu_uart_txd_i(debug_txd_pad),

  .internal_osc_o(internal_osc_o),
  .clk_100mhz_i(pll_100mhz),
  .reboot_o(sc_reboot),
  .led_grn_o(led_grn),
  .led_red_o(led_red),
  .mode_i(mode_q),
  .sd_led_o(sd_led),
  .tx_en_o(tx_en),
  .bbclk_o(bbclk),
  .ts_opt_o(ts_opt),
  .spi_lcd_o(spi_lcd),
  .bus_config_o(bus_config)
);

/****************************************************************************
 * Memory window address decode
 ****************************************************************************/

/* Address decode:
 * 0 to 0xFFFF            16 bit external bus space
 * 0x10000 to 0x1FFFF     8 bit external bus space
 * 0x20000 to 0x200FF     Bus config register and other user registers
 * higher = reserved
 * Memory window currently decodes 27 bits of address.
 *
 * mwinbuswidth settings:
 *     0 - use internal setting
 *     1 - 32 bit
 *     2 - 16 bit
 *     3 - 8 bit
 */

reg [3:0] decodes;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) decodes <= 4'b000;
  else if (mwinwbm_adr_o < 32'h00020000) decodes <= 4'b0001;
  else decodes <= 4'b1000;
end

always @(*) begin
  xbuswbs_en = 1'b0;
  mwinwbm_dat_i = 32'hxxxxxxxx;
  mwinwbm_ack_i = mwinwbm_cyc_o && mwinwbm_stb_o;
  mwinbuswidth = 2'b00;

  if (decodes[0] && xbus_opt) begin
    xbuswbs_en = 1'b1;
    mwinwbm_dat_i[15:0] = xbuswbs_dat_o;
    mwinwbm_ack_i = xbuswbs_ack_o;
  end /*else if (decodes[1]) begin
    mwinwbm_dat_i[15:0] = userwbs_dat_o;
    mwinwbm_ack_i = userwbs_ack_o;
    mwinbuswidth = 2'b10;
  end else if (decodes[2] && adc_opt) begin
    adcwbs_en = 1'b1;
    mwinwbm_dat_i[15:0] = adcwbs_dat_o;
    mwinwbm_ack_i = adcwbs_ack_o;
    mwinbuswidth = 2'b10;
  end*/

end

/****************************************************************************
* NBUS address decode
*****************************************************************************/
always @(*) begin
  xuwbs_en = 1'b0;
  ramwinwbs_en = 1'b0;
  scwbs_en = 1'b0;
  spiwbs_en = 1'b0;
  mwinwbs_en = 1'b0;
  adcwbs_en = 1'b0;
  tswbs_en = 1'b0;
  cpuwbm_ack_i = 1'b1;
  cpuwbm_dat_i = 16'hxxxx;

  if(cpuwbm_adr_o >= 8'h70) begin
    mwinwbs_en = 1'b1;
    cpuwbm_dat_i = mwinwbs_dat_o;
    cpuwbm_ack_i = mwinwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h60) begin
    spiwbs_en = 1'b1;
    cpuwbm_dat_i = spiwbs_dat_o;
    cpuwbm_ack_i = spiwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h5c) begin
    ramwinwbs_en = 1'b1;
    cpuwbm_dat_i = ramwinwbs_dat_o;
    cpuwbm_ack_i = ramwinwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h58) begin
    tswbs_en = 1'b1;
    cpuwbm_dat_i = tswbs_dat_o;
    cpuwbm_ack_i = tswbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h40) begin
    xuwbs_en = 1'b1;
    cpuwbm_dat_i = xuwbs_dat_o;
    cpuwbm_ack_i = xuwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h30) begin
    adcwbs_en = 1'b1;
    cpuwbm_dat_i = adcwbs_dat_o;
    cpuwbm_ack_i = adcwbs_ack_o;
  end else if(cpuwbm_adr_o >= 8'h00) begin
    scwbs_en = 1'b1;
    cpuwbm_dat_i = scwbs_dat_o;
    cpuwbm_ack_i = scwbs_ack_o;
  end

end

endmodule

/* 8to16adapter does one of two things:
 * If toggle8bit_i is 0, it passes the 16 bit wishbone bus straight through.
 * Otherwise, it converts wbm 8 bit cycles into wbs 16 bit cycles.
 */

module wb_8to16adapter (
  wbm_dat_o,
  wbm_dat_i,
  wbm_adr_o,
  wbm_sel_o,

  wbs_dat_i,
  wbs_dat_o,
  wbs_adr_i,
  wbs_sel_i,

  toggle8bit_i
);

input [15:0] wbm_dat_o;
output [15:0] wbm_dat_i;
input [31:0] wbm_adr_o;
input [1:0] wbm_sel_o;

output [15:0] wbs_dat_i;
input [15:0] wbs_dat_o;
output [31:0] wbs_adr_i;
output [1:0] wbs_sel_i;

input toggle8bit_i;

wire t = toggle8bit_i;
wire e = !wbm_adr_o[0];

assign wbm_dat_i = (t && !e) ? {8'hxx, wbs_dat_o[15:8]} : wbs_dat_o;
assign wbs_dat_i = t ? {wbm_dat_o[7:0], wbm_dat_o[7:0]} : wbm_dat_o;
assign wbs_adr_i[31:1] = wbm_adr_o[31:1];
assign wbs_adr_i[0] = t ? 1'b0 : wbm_adr_o[0];
assign wbs_sel_i = t ? e ? 2'b01 : 2'b10 : wbm_sel_o;

endmodule
